package ru.dbotthepony.mc.otm.capability;

import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.LongTag;
import net.minecraft.nbt.Tag;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.food.FoodData;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.CapabilityInject;
import net.minecraftforge.common.capabilities.ICapabilityProvider;
import net.minecraftforge.common.util.INBTSerializable;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public class AndroidCapability implements ICapabilityProvider, IAndroidCapability, INBTSerializable<Tag> {
	private final LivingEntity ent;
	public long energy_stored = 32000L;
	public long energy_stored_max = 32000L;

	public AndroidCapability(LivingEntity ent) {
		this.ent = ent;
	}

	@CapabilityInject(IAndroidCapability.class)
	public static Capability<IAndroidCapability> CAPABILITY = null;

	@Override
	public Tag serializeNBT() {
		CompoundTag tag = new CompoundTag();
		tag.put("energy_stored", LongTag.valueOf(energy_stored));
		tag.put("energy_stored_max", LongTag.valueOf(energy_stored_max));
		return tag;
	}

	@Override
	public void deserializeNBT(Tag nbt) {
		if (nbt instanceof CompoundTag) {
			if (((CompoundTag) nbt).get("energy_stored") != null)
				((CompoundTag) nbt).getLong("energy_stored");

			if (((CompoundTag) nbt).get("energy_stored_max") != null)
				((CompoundTag) nbt).getLong("energy_stored_max");
		}
	}

	@Override
	public void tick() {
		if (ent instanceof ServerPlayer) {
			FoodData stats = ((ServerPlayer) ent).getFoodData();

			if (stats.needsFood()) {
				int food_points_missing = 20 - stats.getFoodLevel();
				boolean food_changed = false;
				float saturation_points_missing = 20F - stats.getSaturationLevel();
				boolean saturation_changed = false;

				while (food_points_missing > 0) {
					if (this.extractEnergy(200L, true) == 200L) {
						this.extractEnergy(200L, false);
						food_points_missing--;
						food_changed = true;
					} else {
						break;
					}
				}

				while (saturation_points_missing >= 1F) {
					if (this.extractEnergy(200L, true) == 200L) {
						this.extractEnergy(200L, false);
						saturation_points_missing--;
						saturation_changed = true;
					} else {
						break;
					}
				}

				if (food_changed) {
					stats.setFoodLevel(20 - food_points_missing);
				}

				if (saturation_changed) {
					stats.setSaturation(20F - saturation_points_missing);
				}

				OverdriveThatMatters.LOGGER.info("New power level {}", getBatteryLevel());
			}
		}
	}

	@Override
	public LivingEntity getEntity() {
		return this.ent;
	}

	@Override
	public long getBatteryLevel() {
		return energy_stored;
	}

	@Override
	public long getMaxBatteryLevel() {
		return energy_stored_max;
	}

	@Override
	public long receiveEnergy(long howMuch, boolean simulate) {
		long final_value = Math.min(energy_stored_max, energy_stored + howMuch);

		if (!simulate) {
			energy_stored = final_value;
		}

		return final_value - energy_stored;
	}

	@Override
	public long extractEnergy(long howMuch, boolean simulate) {
		long final_value = Math.max(0, energy_stored - howMuch);

		if (!simulate) {
			energy_stored = final_value;
		}

		return energy_stored - final_value;
	}

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (cap == AndroidCapability.CAPABILITY) {
			return LazyOptional.of(() -> this).cast();
		}

		return LazyOptional.empty();
	}
}
